/*
** Modular Logfile Analyzer
** Copyright 2000 Jan Kneschke <jan@kneschke.de>
**
** Homepage: http://www.kneschke.de/projekte/modlogan
**

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

**
** $Id: datatype.c,v 1.10 2001/09/21 16:57:39 ostborn Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <zlib.h>
#include <libxml/tree.h>

#include "config.h"
#include "mdatatypes.h"
#include "datatype.h"
#include "misc.h"

/* begin of BrokenLink */

int mdata_BrokenLink_to_xml(gzFile *fd, mdata *data) {
#define WR(x,f) \
	gzprintf(fd, "<%s>%"#f"</%s>\n", #x, data->data.brokenlink.x, #x);
#define WRS(x,f) \
	if (data->data.brokenlink.x != NULL) { \
		char *s = url_encode(data->data.brokenlink.x); \
		gzprintf(fd, "<%s>%"#f"</%s>\n", #x, s, #x); \
		free(s); \
	} else { \
		gzprintf(fd, "<%s />", #x); \
	}
	
	WR(count, d);
	WR(grouped, d);
	WR(timestamp, ld);
	WRS(referrer, s);
#undef WRS
#undef WR
	
	return 0;
}

int mdata_BrokenLink_free(mdata *data) {
	if (!data) return -1;
	if (data->type != M_DATA_TYPE_BROKENLINK) return -1;
	
	if (data->data.brokenlink.referrer)	free(data->data.brokenlink.referrer);
	
	return 0;
}

int mdata_BrokenLink_setdata(mdata *data, const char *str, int count, int type,
	time_t timestamp, const char *referrer) {
	
	assert(str);
	
	data->key	= malloc(strlen(str)+1);
	assert(data->key);
	strcpy(data->key, str);
	
	if (data->type == M_DATA_TYPE_UNSET) 
		data->type		= M_DATA_TYPE_BROKENLINK;
	
	data->data.brokenlink.count	= count;
	data->data.brokenlink.grouped	= type;
	data->data.brokenlink.timestamp = timestamp;
	
	
	if (referrer) {
		data->data.brokenlink.referrer = malloc(strlen(referrer) + 1);
		assert(data->data.brokenlink.referrer);
		strcpy(data->data.brokenlink.referrer, referrer);
	} else {
		data->data.brokenlink.referrer = NULL;
	}
	
	return 0;
}

int mdata_BrokenLink_from_xml(mdata *data, xmlNodePtr node, const char *key) {
	long grouped = 0, count = 0, timestamp = 0;
	char *referrer = NULL, *str;
	int i;
	const mdata_values data_values[] = {
		{ "grouped", M_DATA_FIELDTYPE_LONG, 	&(grouped) },
		{ "count", M_DATA_FIELDTYPE_LONG, 	&(count) },
		{ "timestamp", M_DATA_FIELDTYPE_LONG, 	&(timestamp) },
		{ "referrer", M_DATA_FIELDTYPE_STRING, 	&(referrer) },
		
		{ NULL, M_DATA_FIELDTYPE_UNSET,  NULL}
	};

	while (node) {
		if (node->type != XML_ELEMENT_NODE) {
			node = node->next;
			continue;
		}
		
//		fprintf(stderr, "%s.%d: processing %s\n", __FILE__, __LINE__, node->name);
		
		i = 0;
		while (data_values[i].string) {
			if (!strcmp(data_values[i].string, node->name))
				break;
			i++;
		}

		if (data_values[i].string != NULL) {
			if (mdata_insert_value(data_values[i].dest, data_values[i].type, node->children)) {
				fprintf(stderr, "%s.%d: insert failed for '%s'\n", __FILE__, __LINE__, node->name);
			}
		} else {
			fprintf(stderr, "%s.%d: insert failed, element not found '%s'\n", __FILE__, __LINE__, node->name);
		}
		
		node = node->next;
	}
	
	str = url_decode(referrer);
	
	mdata_BrokenLink_setdata(data, key, count, grouped, timestamp, str);
	
	free(str);
	
	return 0;
}



int mdata_BrokenLink_append(mdata *dst, mdata *src) {
	dst->data.brokenlink.count += src->data.brokenlink.count;
	return M_DATA_APPENDED;
}

mdata *mdata_BrokenLink_init() {
	mdata *data = mdata_init();
	assert(data);
	
	data->key		= NULL;
	data->type		= M_DATA_TYPE_BROKENLINK;
	
#ifdef DEBUG_DATATYPES
	fprintf(stderr, "%s.%d: got %ld (%d)\n", __FILE__, __LINE__, data->id, data->type);
#endif
/* init specifics */	
	data->data.brokenlink.count	= 0;
	data->data.brokenlink.grouped	= M_DATA_STATE_PLAIN;
	data->data.brokenlink.timestamp	= 0;
	data->data.brokenlink.referrer	= NULL;
	
	
	return data;
}

mdata *mdata_BrokenLink_copy(mdata *src) {
	mdata *data = mdata_BrokenLink_init();
	assert(data);
	mdata_BrokenLink_setdata(data, src->key, 
		src->data.brokenlink.count, 
		src->data.brokenlink.grouped,
		src->data.brokenlink.timestamp,
		src->data.brokenlink.referrer
	);
	
	return data;
}

mdata *mdata_BrokenLink_create(const char *str, 
	int count, int grouped,
	time_t timestamp, const char *referrer) {
	mdata *data = mdata_BrokenLink_init();
	assert(data);
	mdata_BrokenLink_setdata(data, str, count, grouped, timestamp, referrer);
	
	return data;
}

/* end of BrokenLink */
