/*
** Modular Logfile Analyzer
** Copyright 2000 Jan Kneschke <jan@kneschke.de>
**
** Homepage: http://www.kneschke.de/projekte/modlogan
**

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

**
** $Id: datatype.c,v 1.7 2001/09/06 15:55:13 ostborn Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <zlib.h>

#include "config.h"
#include "mdatatypes.h"
#include "datatype.h"

/* begin of SubList */

int mdata_SubList_to_xml(gzFile *fd, mdata *data) {
#define WR(x,f) \
	gzprintf(fd, "<%s>%"#f"</%s>\n", #x, data->data.sublist.x, #x);
	
	WR(count, d);
	
	gzprintf(fd, "<sublist>\n");
	mlist_write(fd, data->data.sublist.sublist);
	gzprintf(fd, "</sublist>\n");
#undef WR	
	return 0;
}

int mdata_SubList_free(mdata *data) {
	if (!data) return -1;
	if (data->type != M_DATA_TYPE_SUBLIST) return -1;
	
	if (data->data.sublist.sublist) mlist_free(data->data.sublist.sublist);
	
	return 0;
}

int mdata_SubList_setdata(mdata *data, const char *str, mlist *list, int count) {
	
	data->key	= malloc(strlen(str)+1);
	assert(data->key);
	strcpy(data->key, str);
	
	data->data.sublist.sublist = list ? list : mlist_init();
	data->data.sublist.count   = count;
	
	if (data->type == M_DATA_TYPE_UNSET) 
		data->type = M_DATA_TYPE_SUBLIST;
	
	return 0;
}

int mdata_SubList_from_xml(mdata *data, xmlNodePtr node, const char *key) {
	long count = 0;
	mlist *list = mlist_init();
	int i;
	const mdata_values data_values[] = {
		{ "count", M_DATA_FIELDTYPE_LONG, 	&(count) },
		{ "sublist", M_DATA_FIELDTYPE_LIST, 	&(list) },
		
		{ NULL, M_DATA_FIELDTYPE_UNSET,  NULL}
	};

	while (node) {
		if (node->type != XML_ELEMENT_NODE) {
			node = node->next;
			continue;
		}
		
//		fprintf(stderr, "%s.%d: processing %s\n", __FILE__, __LINE__, node->name);
		
		i = 0;
		while (data_values[i].string) {
			if (!strcmp(data_values[i].string, node->name))
				break;
			i++;
		}

		if (data_values[i].string != NULL) {
			if (mdata_insert_value(data_values[i].dest, data_values[i].type, node->children)) {
				fprintf(stderr, "%s.%d: insert failed for '%s'\n", __FILE__, __LINE__, node->name);
			}
		} else {
			fprintf(stderr, "%s.%d: insert failed, element not found '%s'\n", __FILE__, __LINE__, node->name);
		}
		
		node = node->next;
	}
	
	mdata_SubList_setdata(data, key, list, count);
	
	return 0;
}



int mdata_SubList_append(mdata *dst, mdata *src) {
	dst->data.sublist.count += src->data.sublist.count;
	
	return 0;
}

mdata *mdata_SubList_init() {
	mdata *data = mdata_init();
	assert(data);
	
	data->key		= NULL;
	data->type		= M_DATA_TYPE_SUBLIST;
	
#ifdef DEBUG_DATATYPES
	fprintf(stderr, "%s.%d: got %ld (%d)\n", __FILE__, __LINE__, data->id, data->type);
#endif
/* init specifics */	
	data->data.sublist.sublist	= NULL;
	data->data.sublist.count	= 0;
	
	return data;
}

mdata *mdata_SubList_copy(mdata *src) {
	mdata *data = mdata_SubList_init();
	assert(data);
	mdata_SubList_setdata(data, src->key, src->data.sublist.sublist, src->data.sublist.count);
	
	return data;
}

mdata *mdata_SubList_create(const char *str, mlist *list) {
	mdata *data = mdata_SubList_init();
	assert(data);
	mdata_SubList_setdata(data, str, list, 1);
	
	return data;
}

/* end of SubList */
