/*
** Modular Logfile Analyzer
** Copyright 2000 Jan Kneschke <jan@kneschke.de>
**
** Homepage: http://www.kneschke.de/projekte/modlogan
**

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

**
** $Id: datatype.c,v 1.12 2001/09/21 16:57:39 ostborn Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <zlib.h>

#include "config.h"
#include "mdatatypes.h"
#include "datatype.h"
#include "datatypes/brokenlink/datatype.h"
#include "mlist.h"
#include "misc.h"

/* begin of Visit */

int mdata_Visit_to_xml(gzFile *fd, mdata *data) {
#define WR(x,f) \
	gzprintf(fd, "<%s>%"#f"</%s>\n", #x, data->data.visit.x, #x);
#define WRS(x,f) \
	if (data->data.visit.x != NULL) { \
		char *s = url_encode(data->data.visit.x); \
		gzprintf(fd, "<%s>%"#f"</%s>\n", #x, s, #x); \
		free(s); \
	} else { \
		gzprintf(fd, "<%s />", #x); \
	}
	
	WR(count, d);
	WR(type, d);
	WR(timestamp, ld);
	WR(timediff, ld);
	WRS(useragent, s);
	
	gzprintf(fd, "<hits>\n");
	mlist_write(fd, data->data.visit.hits);
	gzprintf(fd, "</hits>\n");
#undef WRS
#undef WR
	return 0;
}

int mdata_Visit_free(mdata *data) {
	if (!data) return -1;
	if (data->type != M_DATA_TYPE_VISIT) return -1;
	
	if (data->data.visit.useragent) free(data->data.visit.useragent);
	if (data->data.visit.hits) mlist_free(data->data.visit.hits);
	
	return 0;
}

int mdata_Visit_setdata(mdata *data, const char *str, 
	const char *url, const char *useragent,
	int count, time_t timestamp,
	time_t timediff, int type) {
	
	data->key	= malloc(strlen(str)+1);
	assert(data->key);
	strcpy(data->key, str);
	
	if (data->type == M_DATA_TYPE_UNSET) 
		data->type		= M_DATA_TYPE_VISIT;	
	
	data->data.visit.count		= count;
	data->data.visit.type		= type;
	data->data.visit.timestamp	= timestamp;
	data->data.visit.timediff	= timediff;
	
	if (useragent) {
		data->data.visit.useragent	= malloc(strlen(useragent)+1);;
		assert(data->data.visit.useragent);
		strcpy(data->data.visit.useragent, useragent);
	} else {
		data->data.visit.useragent = malloc(strlen("") + 1);
		assert(data->data.visit.useragent);
		strcpy(data->data.visit.useragent, "");
	}
	
	data->data.visit.hits = mlist_init();
	
	if (url) {
		mdata *hit;
		hit = mdata_BrokenLink_create(url, 1, M_DATA_STATE_PLAIN, timestamp, NULL);
	
		mlist_append(data->data.visit.hits, hit);
	}
	
	return 0;
}

int mdata_Visit_from_xml(mdata *data, xmlNodePtr node, const char *key) {
	long grouped = 0, count = 0, type = 0;
	time_t timestamp = 0, timediff = 0;
	char *useragent = NULL, *lasturl = NULL, *str;
	int i;
	
	mlist *hits = mlist_init();
	
	/* add your definitions here */
	const mdata_values data_values[] = {
		{ "count", M_DATA_FIELDTYPE_LONG, 	&(count) },
		{ "type", M_DATA_FIELDTYPE_LONG, 	&(type) },
		{ "timestamp", M_DATA_FIELDTYPE_LONG, 	&(timestamp) },
		{ "timediff", M_DATA_FIELDTYPE_LONG, 	&(timediff) },
		{ "useragent", M_DATA_FIELDTYPE_STRING, &(useragent) },
		{ "lasturl", M_DATA_FIELDTYPE_STRING, 	&(lasturl) }, /* unused */
		{ "hits", M_DATA_FIELDTYPE_LIST,        &(hits) },
		
		{ NULL, M_DATA_FIELDTYPE_UNSET,  NULL}
	};

	while (node) {
		if (node->type != XML_ELEMENT_NODE) {
			node = node->next;
			continue;
		}
		
//		fprintf(stderr, "%s.%d: processing %s\n", __FILE__, __LINE__, node->name);
		
		i = 0;
		while (data_values[i].string) {
			if (!strcmp(data_values[i].string, node->name))
				break;
			i++;
		}

		if (data_values[i].string != NULL) {
			if (mdata_insert_value(data_values[i].dest, data_values[i].type, node->children)) {
				fprintf(stderr, "%s.%d: insert failed for '%s'\n", __FILE__, __LINE__, node->name);
			}
		} else {
			fprintf(stderr, "%s.%d: insert failed, element not found '%s'\n", __FILE__, __LINE__, node->name);
		}
		
		node = node->next;
	}
	
	if (lasturl) free(lasturl);
	
	str = url_decode(useragent);

	mdata_Visit_setdata(data, key, NULL, str, count, timestamp, timediff, grouped);
	
	free(str);

	return 0;
}

int mdata_Visit_append(mdata *dst, mdata *src) {
	return 0;
}

mdata *mdata_Visit_init() {
	mdata *data = mdata_init();
	assert(data);
	
	data->key		= NULL;
	data->type		= M_DATA_TYPE_VISIT;
	
#ifdef DEBUG_DATATYPES
	fprintf(stderr, "%s.%d: got %ld (%d)\n", __FILE__, __LINE__, data->id, data->type);
#endif
/* init specifics */	
	data->data.visit.count		= 0;
	data->data.visit.type		= M_DATA_VISIT_USER;
	data->data.visit.timestamp	= 0;
	data->data.visit.timediff	= 0;
	data->data.visit.useragent	= NULL;
	data->data.visit.hits           = NULL;
	
	return data;
}

mdata *mdata_Visit_copy(mdata *src) {
	mdata *data = mdata_Visit_init();
	assert(data);
	mdata_Visit_setdata(data, src->key, 
		NULL, 
		src->data.visit.useragent,
		src->data.visit.count,
		src->data.visit.timestamp,
		src->data.visit.timediff,
		src->data.visit.type
	);
	
	return data;
}

mdata *mdata_Visit_create(const char *str, 
	const char *url, const char *useragent,
	int count, time_t timestamp,
	time_t timediff, int type
	) {
	
	mdata *data = mdata_Visit_init();
	assert(data);

	mdata_Visit_setdata(data, str, url,
		useragent, count, timestamp, 
		timediff, type);
	
	return data;
}

/* end of Visit */
