/*
** Modular Logfile Analyzer
** Copyright 2000 Jan Kneschke <jan@kneschke.de>
**
** Homepage: http://www.kneschke.de/projekte/modlogan
**

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

**
** $Id: datatype.c,v 1.5 2001/09/06 15:55:13 ostborn Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <zlib.h>
#include <libxml/tree.h>

#include "config.h"
#include "mdatatypes.h"
#include "datatype.h"

/* begin of Visited */

int mdata_Visited_to_xml(gzFile *fd, mdata *data) {
#define WR(x,f) \
	gzprintf(fd, "<%s>%"#f"</%s>\n", #x, data->data.visited.x, #x);
	
	WR(count, d);
	WR(grouped, d);
	WR(vcount, d);
#undef WR
	return 0;
}

int mdata_Visited_free(mdata *data) {
	if (!data) return -1;
	if (data->type != M_DATA_TYPE_VISITED) return -1;
	
	return 0;
}

int mdata_Visited_setdata(mdata *data, const char *str, int count, int type, int vcount) {
	if (data == NULL || str == NULL) {
		fprintf(stderr, "%s.%d: (visited) either data (%p) or str (%p) is NULL\n",
			__FILE__,
			__LINE__,
			data,
			str);
		
		return -1;
	}
	
	
	data->key	= malloc(strlen(str)+1);
	assert(data->key);
	strcpy(data->key, str);
	
	if (data->type == M_DATA_TYPE_UNSET) 
		data->type		= M_DATA_TYPE_VISITED;
	
	data->data.visited.count	= count;
	data->data.visited.grouped	= type;
	data->data.visited.vcount	= vcount;
	
	return 0;
}

int mdata_Visited_from_xml(mdata *data, xmlNodePtr node, const char *key) {
	long grouped = 0, count = 0, visited = 0;
	int i;
	const mdata_values data_values[] = {
		{ "grouped", M_DATA_FIELDTYPE_LONG, 	&(grouped) },
		{ "count", M_DATA_FIELDTYPE_LONG, 	&(count) },
		{ "vcount", M_DATA_FIELDTYPE_LONG, 	&(visited) },
		
		{ NULL, M_DATA_FIELDTYPE_UNSET,  NULL}
	};

	while (node) {
		if (node->type != XML_ELEMENT_NODE) {
			node = node->next;
			continue;
		}
		
//		fprintf(stderr, "%s.%d: processing %s\n", __FILE__, __LINE__, node->name);
		
		i = 0;
		while (data_values[i].string) {
			if (!strcmp(data_values[i].string, node->name))
				break;
			i++;
		}

		if (data_values[i].string != NULL) {
			if (mdata_insert_value(data_values[i].dest, data_values[i].type, node->children)) {
				fprintf(stderr, "%s.%d: (visited) insert failed for '%s'\n", __FILE__, __LINE__, node->name);
			}
		} else {
			fprintf(stderr, "%s.%d: (visited) insert failed, element not found '%s'\n", __FILE__, __LINE__, node->name);
		}
		
		node = node->next;
	}
	
	mdata_Visited_setdata(data, key, count, grouped, visited);

	return 0;
}



int mdata_Visited_append(mdata *dst, mdata *src) {
	dst->data.visited.count += src->data.visited.count;
	dst->data.visited.vcount += src->data.visited.vcount;
	
	return 0;
}

mdata *mdata_Visited_init() {
	mdata *data = mdata_init();
	assert(data);
	
	data->key		= NULL;
	data->type		= M_DATA_TYPE_VISITED;
	
#ifdef DEBUG_DATATYPES
	fprintf(stderr, "%s.%d: got %ld (%d)\n", __FILE__, __LINE__, data->id, data->type);
#endif
/* init specifics */	
	data->data.visited.count	= 0;
	data->data.visited.grouped	= M_DATA_STATE_PLAIN;
	data->data.visited.vcount	= 0;
	
	return data;
}

mdata *mdata_Visited_copy(mdata *src) {
	mdata *data = mdata_Visited_init();
	
	mdata_Visited_setdata(data, 
		src->key, 
		src->data.visited.count, 
		src->data.visited.grouped,
		src->data.visited.vcount);
	
	return data;
}

mdata *mdata_Visited_create(const char *str, int count, int type, int vcount) {
	mdata *data = mdata_Visited_init();
	
	mdata_Visited_setdata(data, str, count, type, vcount);
	
	return data;
}

/* end of Visited */
