/*
** Modular Logfile Analyzer
** Copyright 2000 Jan Kneschke <jan@kneschke.de>
**
** Homepage: http://www.kneschke.de/projekte/modlogan
**

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

**
** $Id: parse.c,v 1.3 2001/09/09 14:43:44 ostborn Exp $
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>

#include "mlocale.h"
#include "mplugins.h"
#include "mrecord.h"
#include "mdatatypes.h"
#include "misc.h"

#include "plugin_config.h"

#define DEBUG_PCRE

typedef struct {
	int type;
	pcre *match;
} Matches;

int parse_record_pcre(mconfig *ext_conf, mlogrec *record, char *_buffer) {
#define N 20 + 1
	const char **list;
	int ovector[3 * N], n;
	
	int i, match;

	enum {
		M_BSDFTPD_USER_LOGIN_SUCCESS, M_BSDFTPD_CONNECTION, M_BSDFTPD_RENAME, M_BSDFTPD_MKDIR, 
			M_BSDFTPD_USER, M_BSDFTPD_GET, M_BSDFTPD_PUT,
			M_BSDFTPD_DELETE, M_BSDFTPD_RMDIR, M_BSDFTPD_APPEND,
			M_BSDFTPD_USER_LOGIN_FAILED,M_BSDFTPD_USER_LOGIN_REFUSED,
			M_BSDFTPD_ANONYMOUS_LOGIN_SUCCESS,M_BSDFTPD_ANONYMOUS_LOGIN_REFUSED
	};
	
	config_input *conf = ext_conf->plugin_conf;
	
	const Matches matches [] = { 
		{ M_BSDFTPD_USER_LOGIN_SUCCESS, conf->match_bsdftpd_user_login_success },
		{ M_BSDFTPD_CONNECTION, conf->match_bsdftpd_connection },
		{ M_BSDFTPD_RENAME, conf->match_bsdftpd_rename },
		{ M_BSDFTPD_MKDIR, conf->match_bsdftpd_mkdir },
		{ M_BSDFTPD_USER, conf->match_bsdftpd_user },
		{ M_BSDFTPD_GET, conf->match_bsdftpd_get },
		{ M_BSDFTPD_PUT, conf->match_bsdftpd_put },
		{ M_BSDFTPD_DELETE, conf->match_bsdftpd_delete },
		{ M_BSDFTPD_RMDIR, conf->match_bsdftpd_rmdir },
		{ M_BSDFTPD_APPEND, conf->match_bsdftpd_append },
		{ M_BSDFTPD_USER_LOGIN_FAILED, conf->match_bsdftpd_user_login_failed },
		{ M_BSDFTPD_USER_LOGIN_REFUSED, conf->match_bsdftpd_user_login_refused },
		{ M_BSDFTPD_ANONYMOUS_LOGIN_SUCCESS, conf->match_bsdftpd_anonymous_login_success },
		{ M_BSDFTPD_ANONYMOUS_LOGIN_REFUSED, conf->match_bsdftpd_anonymous_login_refused },
		
		
		{ 0, NULL } };

	/* try to match the syslog prefix */

	if ((n = pcre_exec(conf->match_bsdftpd_syslog_pre, NULL, _buffer, strlen(_buffer), 0, 0, ovector, 3 * N)) < 0) {
		if (n == PCRE_ERROR_NOMATCH) {
//			fprintf(stderr, "%s.%d: syslog prefix doesn't match: %s\n", __FILE__, __LINE__, _buffer);
			return M_RECORD_IGNORED;
		} else {
			fprintf(stderr, "%s.%d: execution error while matching: %d\n", __FILE__, __LINE__, n);
			return M_RECORD_HARD_ERROR;
		}
	}
	
	/* ok, it's a syslog - the rest should be qmail specific */
	if (n) {
		i = 0;
		match = -1;
		while ( matches[i].match != NULL ) {
		/* find the corresponding match */
			if ((n = pcre_exec(matches[i].match, NULL, _buffer, strlen(_buffer), 0, 0, ovector, 3 * N)) < 0) {
				if (n == PCRE_ERROR_NOMATCH) {
					i++;
					continue;
				} else {
					fprintf(stderr, "%s.%d: execution error while matching: %d\n", __FILE__, __LINE__, n);
					return M_RECORD_HARD_ERROR;
				}
			} else {
				match = matches[i].type;
				break;
			}
		}
		
		if (n > 1 && match != -1) {
			pcre_get_substring_list(_buffer, ovector, n, &list);
			
			record->timestamp = strtol(list[1], NULL, 10);
			
			switch (match) {
			case M_BSDFTPD_CONNECTION:
			case M_BSDFTPD_USER_LOGIN_SUCCESS:
			case M_BSDFTPD_USER_LOGIN_REFUSED:
			case M_BSDFTPD_USER_LOGIN_FAILED:
			case M_BSDFTPD_ANONYMOUS_LOGIN_SUCCESS:
			case M_BSDFTPD_ANONYMOUS_LOGIN_REFUSED:
			case M_BSDFTPD_RENAME:
			case M_BSDFTPD_MKDIR:
			case M_BSDFTPD_USER:
			case M_BSDFTPD_GET:
			case M_BSDFTPD_PUT:
			case M_BSDFTPD_RMDIR:
			case M_BSDFTPD_DELETE:
			case M_BSDFTPD_APPEND:
				break;
			default:
#if 1
				for (i = 0; i < n; i++) {
					printf("%d: %s\n", i, list[i]);
				}
#endif
			}
			free(list);
			
			return record->ext ? M_RECORD_NO_ERROR : M_RECORD_IGNORED;
		} else {
			fprintf(stderr, "%s.%d: was das ?? %s\n", __FILE__, __LINE__, _buffer);
			return M_RECORD_CORRUPT;
		}
	} else {
		return M_RECORD_CORRUPT;
	}

	return M_RECORD_IGNORED;
#undef  N
}


int mplugins_input_bsdftpd_get_next_record(mconfig *ext_conf, mlogrec *record) {
	config_input *conf = ext_conf->plugin_conf;
	
	if (!fgets(conf->buffer, conf->buf_len-1,conf->inputfile)) {
		return M_RECORD_EOF;
	}

	while (conf->buffer[strlen(conf->buffer)-1] != '\n') {
		conf->buffer = realloc(conf->buffer, (conf->buf_len+conf->buf_inc) * sizeof(char));
		
		if (!fgets(conf->buffer+strlen(conf->buffer), conf->buf_inc-1,conf->inputfile)) {
			return M_RECORD_EOF;
		}
		
		conf->buf_len += conf->buf_inc;
	}
	return parse_record_pcre(ext_conf, record, conf->buffer);
}
