/*
** Modular Logfile Analyzer
** Copyright 2000 Jan Kneschke <jan@kneschke.de>
**
** Homepage: http://www.kneschke.de/projekte/modlogan
**

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

**
** $Id: plugin_config.c,v 1.7 2001/09/08 17:13:01 ostborn Exp $
*/

#include <libintl.h>
#include <locale.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <limits.h>

#include "mconfig.h"
#include "mstate.h"
#include "mlocale.h"
#include "mhash.h"
#include "mlist.h"
#include "mdatatypes.h"
#include "mplugins.h"

#include "plugin_config.h"

int mplugins_output_template_dlinit(mconfig *ext_conf) {
	config_output *conf = NULL;
	
	conf = malloc(sizeof(config_output));
	
	memset(conf, 0, sizeof(config_output));
	
	conf->col_circle = mlist_init();
	
	ext_conf->plugin_conf = conf;
	
	return 0;
}

int mplugins_output_template_dlclose(mconfig *ext_conf) {
	config_output *conf = ext_conf->plugin_conf;
	
	if (conf->assumedprotocol)	free(conf->assumedprotocol);
	if (conf->hostname)	free(conf->hostname);
	
	mlist_free(conf->col_circle);

	free(ext_conf->plugin_conf);
	ext_conf->plugin_conf = NULL;
	
	return 0;
}

int mplugins_output_template_parse_config(mconfig *ext_conf, const char *filename, const char *section) {
	config_output *conf = ext_conf->plugin_conf;
	
	const mconfig_values config_values[] = {
	/* mlist - StrInt */
		{"country_circle",	M_CONFIG_TYPE_STRING_LIST, M_CONFIG_VALUE_APPEND, &(conf->col_circle)},
		{"dont_escape_entities", M_CONFIG_TYPE_INT, M_CONFIG_VALUE_OVERWRITE, &(conf->dont_escape_entities)},
		{"dont_cut_urls", M_CONFIG_TYPE_INT,	M_CONFIG_VALUE_OVERWRITE, &(conf->dont_cut_urls)},
	/* strings */	
		{"assumedprotocol", M_CONFIG_TYPE_STRING, M_CONFIG_VALUE_OVERWRITE, &(conf->assumedprotocol)},
		{"hostname", M_CONFIG_TYPE_STRING,	M_CONFIG_VALUE_OVERWRITE, &(conf->hostname)},
		{"content_language", M_CONFIG_TYPE_STRING, M_CONFIG_VALUE_OVERWRITE, &(conf->cont_language)},
		{"content_charset", M_CONFIG_TYPE_STRING, M_CONFIG_VALUE_OVERWRITE, &(conf->cont_charset)},
		
		{NULL, M_CONFIG_TYPE_INT, 0, NULL}
	};
	
	return mconfig_parse_section(ext_conf, filename, section, config_values);
}

int mplugins_output_template_set_defaults(mconfig *ext_conf) {
	config_output *conf = ext_conf->plugin_conf;
#define LOCALHOST	"localhost"
#define CONT_LANG	"en"
#define CONT_CHARSET	"iso-8859-1"
	
	if (conf->hostname == NULL) {
		conf->hostname = malloc(strlen(LOCALHOST)+1);
		strcpy(conf->hostname, LOCALHOST);
	}
	
	if (conf->cont_language == NULL) {
		conf->cont_language = malloc(strlen(CONT_LANG)+1);
		strcpy(conf->cont_language, CONT_LANG);
	}
	
	if (conf->cont_charset == NULL) {
		conf->cont_charset = malloc(strlen(CONT_CHARSET)+1);
		strcpy(conf->cont_charset, CONT_CHARSET);
	}
	
	return 0;
}

int mplugins_init(mplugin *func) {
	func->dlinit = mplugins_output_template_dlinit;
	func->dlclose = mplugins_output_template_dlclose;
	func->parse_config = mplugins_output_template_parse_config;
	func->set_defaults = mplugins_output_template_set_defaults;
	func->get_next_record = NULL;
	func->insert_record = NULL;
	func->gen_report = NULL;
        func->gen_history = NULL;
	
	return 0;
}
